@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2016-2021 European Commission (JRC);
+# Copyright 2016-2022 European Commission (JRC);
 # Licensed under the EUPL (the 'Licence');
 # You may not use this work except in compliance with the Licence.
 # You may obtain a copy of the Licence at: http://ec.europa.eu/idabc/eupl
@@ -23,11 +23,219 @@ def inp_ranges(*rng):
 @ddt.ddt
 class TestCell(unittest.TestCase):
     @ddt.idata([
+        ('A1', '=ADDRESS(2,3,2,FALSE)', {}, '<Ranges>(A1)=[[\'R2C[3]\']]'),
+        ('A1', '=ADDRESS(2,3,2)', {}, '<Ranges>(A1)=[[\'C$2\']]'),
+        ('A1', '=ADDRESS(2,3)', {}, '<Ranges>(A1)=[[\'$C$2\']]'),
+        ('A1', '=ADDRESS(1,1,1, FALSE,"sss\'")', {},
+         '<Ranges>(A1)=[["\'sss\'\'\'!R1C1"]]'),
+        ('A1', '=LARGE({-1.1,4.1,#REF!},1)', {}, '<Ranges>(A1)=[[#REF!]]'),
+        ('A1', '=LARGE({-1.1,4.1,#REF!},"c")', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=LARGE({-1.1,4.1,#REF!},#N/A)', {}, '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=LARGE(#N/A,#VALUE!)', {}, '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=SLOPE(A2,{1})', {'A2': [[sh.EMPTY]]},
+         '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=FORECAST(32,{2,3,4},{1,1,1})', {}, '<Ranges>(A1)=[[#DIV/0!]]'),
+        ('A1', '=NPER(0,4,5)', {}, '<Ranges>(A1)=[[-1.25]]'),
+        ('A1', '=FV(0.06,-10,-200,-500,1)', {},
+         '<Ranges>(A1)=[[-1281.1410664423581]]'),
+        ('A1', '=FV(0.06,-10,200,-500,1)', {},
+         '<Ranges>(A1)=[[1839.5358433574759]]'),
+        ('A1', '=CUMIPMT(0.09/12,30*12,125000,13,24,4)', {},
+         '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=CUMIPMT(0.09/12,30*12,125000,13,24,0)', {},
+         '<Ranges>(A1)=[[-11135.232130750845]]'),
+        ('A1', '=CUMIPMT(3,5.5,125000,3,5,1)', {},
+         '<Ranges>(A1)=[[-265999.0229604315]]'),
+        ('A1', '=CUMIPMT(3,5.5,125000,1,5,1)', {},
+         '<Ranges>(A1)=[[-359611.6267708859]]'),
+        ('A1', '=CUMIPMT(3,5.5,125000,1,6,1)', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=RATE(1500,-200,8000)', {},
+         '<Ranges>(A1)=[[0.025000000047328684]]'),
+        ('A1', '=RATE(43,-200,8000)', {},
+         '<Ranges>(A1)=[[0.0033315518252391286]]'),
+        ('A1', '=PV(100,3,22.4)', {}, '<Ranges>(A1)=[[-0.22399978258780684]]'),
+        ('A1', '=PV(100,3.1,22)', {}, '<Ranges>(A1)=[[-0.21999986540577623]]'),
+        ('A1', '=PV(100,3,22)', {}, '<Ranges>(A1)=[[-0.21999978647016746]]'),
+        ('A1', '=DATEDIF("01/01/01","06/02/08","YD")', {},
+         '<Ranges>(A1)=[[152]]'),
+        ('A1', '=DATEDIF("01/09/01","06/02/08","YD")', {},
+         '<Ranges>(A1)=[[144]]'),
+        ('A1', '=DATEDIF("01/31/2001","01/29/2008","YD")', {},
+         '<Ranges>(A1)=[[363]]'),
+        ('A1', '=DATEDIF("01/01/2001","02/29/2008","YD")', {},
+         '<Ranges>(A1)=[[59]]'),
+        ('A1', '=DATEDIF("01/01/2001","03/01/2008","YD")', {},
+         '<Ranges>(A1)=[[59]]'),
+        ('A1', '=DATEDIF("01/09/00","06/02/08","YD")', {},
+         '<Ranges>(A1)=[[145]]'),
+        ('A1', '=DATEDIF("01/31/2001","03/29/2007","YD")', {},
+         '<Ranges>(A1)=[[57]]'),
+        ('A1', '=DATEDIF("01/31/01","02/29/08","YM")', {},
+         '<Ranges>(A1)=[[0]]'),
+        ('A1', '=DATEDIF(1, 0, "YD")', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=DATEDIF(-1, 222222, "YD")', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=DATEDIF(-1, 222222, "D")', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=DATEDIF(0, 222222, "YD")', {}, '<Ranges>(A1)=[[154]]'),
+        ('A1', '=DATEDIF(1, 222222, "YD")', {}, '<Ranges>(A1)=[[153]]'),
+        ('A1', '=DATEDIF("01/09/01","02/29/08","YD")', {},
+         '<Ranges>(A1)=[[51]]'),
+        ('A1', '=DATEDIF(60,222222,"YD")', {}, '<Ranges>(A1)=[[94]]'),
+        ('A1', '=DATEDIF(222,222290,"YD")', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=DATEDIF(222,222289,"YD")', {}, '<Ranges>(A1)=[[364]]'),
+        ('A1', '=DATEDIF(222,222222,"YD")', {}, '<Ranges>(A1)=[[297]]'),
+        ('A1', '=DATEDIF(222,222290,"YM")', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=DATEDIF(222,222289,"YM")', {}, '<Ranges>(A1)=[[11]]'),
+        ('A1', '=DATEDIF(222,222222,"YM")', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=DATEDIF(222,222290,"MD")', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=DATEDIF(222,222289,"MD")', {}, '<Ranges>(A1)=[[30]]'),
+        ('A1', '=DATEDIF(222,222222,"MD")', {}, '<Ranges>(A1)=[[24]]'),
+        ('A1', '=DATEDIF(222,222290,"Y")', {}, '<Ranges>(A1)=[[608]]'),
+        ('A1', '=DATEDIF(222,222289,"Y")', {}, '<Ranges>(A1)=[[607]]'),
+        ('A1', '=DATEDIF(222,222222,"Y")', {}, '<Ranges>(A1)=[[607]]'),
+        ('A1', '=DATEDIF(222,222290,"M")', {}, '<Ranges>(A1)=[[7296]]'),
+        ('A1', '=DATEDIF(222,222289,"M")', {}, '<Ranges>(A1)=[[7295]]'),
+        ('A1', '=DATEDIF(222,222222,"M")', {}, '<Ranges>(A1)=[[7293]]'),
+        ('A1', '=DATEDIF(222,222290,"D")', {}, '<Ranges>(A1)=[[222068]]'),
+        ('A1', '=DATEDIF(222,222289,"D")', {}, '<Ranges>(A1)=[[222067]]'),
+        ('A1', '=DATEDIF(222,222222,"D")', {}, '<Ranges>(A1)=[[222000]]'),
+        ('A1', '=WEEKNUM(0, 21)', {}, '<Ranges>(A1)=[[52]]'),
+        ('A1', '=WEEKNUM(1, 21)', {}, '<Ranges>(A1)=[[52]]'),
+        ('A1', '=WEEKNUM(2, 21)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(8, 21)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(15, 21)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(22, 21)', {}, '<Ranges>(A1)=[[3]]'),
+        ('A1', '=WEEKNUM(29, 21)', {}, '<Ranges>(A1)=[[4]]'),
+        ('A1', '=WEEKNUM(9, 21)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(16, 21)', {}, '<Ranges>(A1)=[[3]]'),
+        ('A1', '=WEEKNUM(23, 21)', {}, '<Ranges>(A1)=[[4]]'),
+        ('A1', '=WEEKNUM(30, 21)', {}, '<Ranges>(A1)=[[5]]'),
+        ('A1', '=WEEKNUM(9, 21)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(64, 21)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(65, 21)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(0, 11)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 12)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 13)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 14)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 15)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 16)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(0, 17)', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=WEEKNUM(0, 1)', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=WEEKNUM(0, 2)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(60, 21)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 11)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 12)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 13)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 14)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 15)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 16)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 17)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 1)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 2)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(11690, 21)', {}, '<Ranges>(A1)=[[53]]'),
+        ('A1', '=WEEKNUM(12055, 21)', {}, '<Ranges>(A1)=[[52]]'),
+        ('A1', '=WEEKNUM(12056, 21)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12057, 21)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12425, 11)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12425, 12)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12425, 13)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12425, 14)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12425, 15)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12425, 16)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12425, 17)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12425, 1)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12425, 2)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(60, 11)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 12)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 13)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(60, 14)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 15)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 16)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 17)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 1)', {}, '<Ranges>(A1)=[[9]]'),
+        ('A1', '=WEEKNUM(60, 2)', {}, '<Ranges>(A1)=[[10]]'),
+        ('A1', '=WEEKNUM(11326)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(36534)', {}, '<Ranges>(A1)=[[3]]'),
+        ('A1', '=WEEKNUM(36527)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(12420)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(12421)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(11323)', {}, '<Ranges>(A1)=[[53]]'),
+        ('A1', '=WEEKNUM(11325)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(11327)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(36533)', {}, '<Ranges>(A1)=[[2]]'),
+        ('A1', '=WEEKNUM(36526)', {}, '<Ranges>(A1)=[[1]]'),
+        ('A1', '=WEEKNUM(36525)', {}, '<Ranges>(A1)=[[53]]'),
+        ('A1', '=WEEKDAY(36525)', {}, '<Ranges>(A1)=[[6]]'),
+        ('A1', '=WEEKDAY(0,17)', {}, '<Ranges>(A1)=[[7]]'),
+        ('A1', '=WEEKDAY("a",FALSE)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=WEEKDAY(0,FALSE)', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=WEEKDAY(0,11)', {}, '<Ranges>(A1)=[[6]]'),
+        ('A1', '=WEEKDAY("2.9", "3.9")', {}, '<Ranges>(A1)=[[0]]'),
+        ('A1', '=WEEKDAY(TRUE, 2)', {}, '<Ranges>(A1)=[[7]]'),
+        ('A1', '=WEEKDAY(DATE(1008,5,23))', {}, '<Ranges>(A1)=[[4]]'),
+        ('A1', '=CUMIPMT(0.09/12,30*12,125000,13,24,TRUE)', {},
+         '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=PRODUCT({3,4},{8,9})', {}, '<Ranges>(A1)=[[864.0]]'),
+        ('A1', '=SLOPE({3,4},{8,9})', {}, '<Ranges>(A1)=[[1.0]]'),
+        ('A1', '=CORREL(A2:A4,{8,2})', {'A2:A4': [[sh.EMPTY] * 3]},
+         '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=CORREL({"2",4,1},{7,"",TRUE})', {},
+         '<Ranges>(A1)=[[#DIV/0!]]'),
+        ('A1', '=CORREL(A2:A3,{8,2})', {'A2:A3': [[sh.EMPTY] * 2]},
+         '<Ranges>(A1)=[[#DIV/0!]]'),
+        ('A1', '=CORREL(A2,{8,2})', {'A2': [[sh.EMPTY]]},
+         '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=CORREL({"2",4,1},{7,3,6})', {}, '<Ranges>(A1)=[[-1.0]]'),
+        ('A1', '=CORREL({2,4,1},{7,3,6})', {},
+         '<Ranges>(A1)=[[-0.8386278693775346]]'),
+        ('A1', '=NPER(0.12/12, -100, -1000, 10000, 1)', {},
+         '<Ranges>(A1)=[[59.67386567429457]]'),
+        ('A1', '=NPER(0.12/12, -100, -1000)', {},
+         '<Ranges>(A1)=[[-9.578594039813161]]'),
+        ('A1', '=IPMT(0.1/12, "01/01/1900", 3*12, 8000)', {},
+         '<Ranges>(A1)=[[-66.66666666666667]]'),
+        ('A1', '=IPMT(0.1/12, 60, 3*12, 8000)', {},
+         '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=FV(0.06/12, 10, -200, -500, 1)', {},
+         '<Ranges>(A1)=[[2581.4033740601367]]'),
+        ('A1', '=RATE(4*12, -200, 8000)', {},
+         '<Ranges>(A1)=[[0.00770147248823337]]'),
+        ('A1', '=PV(0.08/12, 12*20, 500000,,0)', {},
+         '<Ranges>(A1)=[[-59777145.85118777]]'),
+        ('A1', '=PPMT(0.1/12, 1, 2*12, 2000)', {},
+         '<Ranges>(A1)=[[-75.62318600836664]]'),
+        ('A1', '=PMT("33","3","33","10000","1")', {},
+         '<Ranges>(A1)=[[-32.2771768657085]]'),
+        ('A1', '=PMT(0.08/12,0,10000)', {}, '<Ranges>(A1)=[[#NUM!]]'),
+        ('A1', '=PMT(0.08/12,10,10000,10000,1)', {},
+         '<Ranges>(A1)=[[-1994.1034887930407]]'),
+        ('A1', '=PMT(0.08/12,10,10000,,1)', {},
+         '<Ranges>(A1)=[[-1030.1643271779772]]'),
+        ('A1', '=PMT(0.08/12,10,10000)', {},
+         '<Ranges>(A1)=[[-1037.0320893591636]]'),
+        ('A1', '=FORECAST(1,A2,{8,2})', {'A2': [[sh.EMPTY]]},
+         '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=FORECAST(1,A2:A3,A2:A3)', {'A2:A3': [[sh.EMPTY, sh.EMPTY]]},
+         '<Ranges>(A1)=[[#DIV/0!]]'),
+        ('A1', '=FORECAST(A2,{3,4},{8,9})', {'A2': [[sh.EMPTY]]},
+         '<Ranges>(A1)=[[-5.0]]'),
+        ('A1', '=FORECAST(#N/A,{"3"},{9})', {}, '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=FORECAST(5,{3},{9})', {}, '<Ranges>(A1)=[[#DIV/0!]]'),
+        ('A1', '=FORECAST(5,{3},{8,9})', {}, '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=FORECAST(2,A2:A5,{7,TRUE,8,9})',
+         {'A2:A5': [[sh.EMPTY, 4, 3, 4]]}, '<Ranges>(A1)=[[-3.0]]'),
+        ('A1', '=FORECAST(2,{"1",4,3,4},{7,TRUE,8,9})', {},
+         '<Ranges>(A1)=[[-3.0]]'),
+        ('A1', '=FORECAST(2,{"1","4",3,4},{7,4,8,9})', {},
+         '<Ranges>(A1)=[[-3.0]]'),
+        ('A1', '=FORECAST(5,{"a",2,3}, {7,8,9})', {}, '<Ranges>(A1)=[[-1.0]]'),
+        ('A1', '=FORECAST(5,{1,2,3}, {7,8,9})', {}, '<Ranges>(A1)=[[-1.0]]'),
         ('A1', '=SEARCH(5,45)', {}, '<Ranges>(A1)=[[2]]'),
         ('A1', '=SEARCH("n", "printer")', {}, '<Ranges>(A1)=[[4]]'),
         ('A1', '=SEARCH("BASE","database",5.8)', {}, '<Ranges>(A1)=[[5]]'),
-        ('A1', '=SEARCH("BASE","database",6.1)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
-        ('A1', '=SEARCH("BASE","database",FALSE)', {}, '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=SEARCH("BASE","database",6.1)', {},
+         '<Ranges>(A1)=[[#VALUE!]]'),
+        ('A1', '=SEARCH("BASE","database",FALSE)', {},
+         '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=SEARCH("BASE","database",TRUE)', {}, '<Ranges>(A1)=[[5]]'),
         ('A1', '=SEARCH("x","database")', {}, '<Ranges>(A1)=[[#VALUE!]]'),
         ('A1', '=EDATE(0, 0)', {}, '<Ranges>(A1)=[[0]]'),
@@ -37,14 +245,14 @@ class TestCell(unittest.TestCase):
         ('A1', '=EDATE("15-Jan-11", 1)', {}, '<Ranges>(A1)=[[40589]]'),
         ('A1', '=EDATE(1, "2")', {}, '<Ranges>(A1)=[[61]]'),
         ('A1', '=EDATE(2.4, 1)', {}, '<Ranges>(A1)=[[33]]'),
-        ('A1', '=A2 %', {'A2':[[20]]}, '<Ranges>(A1)=[[0.2]]'),
-        ('A3', '=SINGLE(F1:F3)', {'F1:F3':[[1], [2], [3]]},
+        ('A1', '=A2 %', {'A2': [[20]]}, '<Ranges>(A1)=[[0.2]]'),
+        ('A3', '=SINGLE(F1:F3)', {'F1:F3': [[1], [2], [3]]},
          '<Ranges>(A3)=[[3]]'),
-        ('A3', '=COLUMN(SINGLE(F1:F3))', {'F1:F3':[[1], [2], [3]]},
+        ('A3', '=COLUMN(SINGLE(F1:F3))', {'F1:F3': [[1], [2], [3]]},
          '<Ranges>(A3)=[[6]]'),
-        ('A4', '=COLUMN(SINGLE(F1:F3))', {'F1:F3':[[1], [2], [3]]},
+        ('A4', '=COLUMN(SINGLE(F1:F3))', {'F1:F3': [[1], [2], [3]]},
          '<Ranges>(A4)=[[#VALUE!]]'),
-        ('A4', '=SINGLE(F1:F3)', {'F1:F3':[[1], [2], [3]]},
+        ('A4', '=SINGLE(F1:F3)', {'F1:F3': [[1], [2], [3]]},
          '<Ranges>(A4)=[[#VALUE!]]'),
         ('A1', '=__xludf.DUMMYFUNCTION("(GOOGLEFINANCE($A1,""Name""))")', {},
          "<Ranges>(A1)=[[#NAME?]]"),
@@ -83,8 +291,6 @@ class TestCell(unittest.TestCase):
         }, '<Ranges>(A1:B1)=[[4.0 #N/A]]'),
         ('A1', '=SMALL({-1.1,10.1;4.1,"40"},4)', {}, '<Ranges>(A1)=[[#NUM!]]'),
         ('A1', '=SMALL({-1.1,10.1;"40",TRUE},2)', {}, '<Ranges>(A1)=[[10.1]]'),
-        ('A1', '=LARGE({-1.1,4.1,#REF!},"c")', {}, '<Ranges>(A1)=[[#VALUE!]]'),
-        ('A1', '=LARGE({-1.1,4.1,#REF!},#N/A)', {}, '<Ranges>(A1)=[[#N/A]]'),
         ('A1', '=LARGE({-1.1,10.1;4.1,"40"},4)', {}, '<Ranges>(A1)=[[#NUM!]]'),
         ('A1', '=LARGE({-1.1,10.1;"40",-2},2)', {}, '<Ranges>(A1)=[[-1.1]]'),
         ('A1', '=LOOKUP(2,{-1.1,2.1,3.1,4.1},{#REF!,2.1,3.1,4.1})', {},
@@ -103,8 +309,8 @@ class TestCell(unittest.TestCase):
          '<Ranges>(A1)=[[1937.5566679300437]]'),
         ('A1', '=XIRR({-10000,2750,3250,4250},{39448,39508,39859,39751}, 1)',
          {}, '<Ranges>(A1)=[[0.03379137764432629]]'),
-        ('A1', '=IRR({-7,2,-1,4,-3;7,2,-1,4,4},A2:C2)',
-         {'A2:C2': [[1, 20, 4]]}, '<Ranges>(A1)=[[0.19086464188385843]]'),
+        ('A1', '=IRR({-7,2,-1,4,-3;7,2,-1,4,4},1)', {},
+         '<Ranges>(A1)=[[0.19086464188385843]]'),
         ('A1', '=IRR({2,0,4},A2)',
          {'A2': [[sh.EMPTY]]}, '<Ranges>(A1)=[[#NUM!]]'),
         ('A1', '=IRR({7,2,-1,4,-3;7,2,-1,4,4},A2)',
@@ -113,8 +319,7 @@ class TestCell(unittest.TestCase):
          {}, '<Ranges>(A1)=[[100.10102845727761]]'),
         ('A1', '=XNPV(0.02,{-10000,2750,3250,4250},{0,39508,39859,39751})',
          {}, '<Ranges>(A1)=[[#NUM!]]'),
-        ('A1', '=NPV(A2:B2, A3:C3)',
-         {'A2:B2': [[1, "ciao"]], 'A3:C3': [[5, 4, 3]]},
+        ('A1', '=NPV(1, A3:C3)', {'A3:C3': [[5, 4, 3]]},
          '<Ranges>(A1)=[[3.875]]'),
         ('A1', '=NPV(D2, A2:C2)',
          {'D2': [["ciao"]], 'A2:C2': [[5, sh.EMPTY, Error.errors['#N/A']]]},
@@ -275,10 +480,14 @@ class TestCell(unittest.TestCase):
         ('A1', '=IF(FALSE, #VALUE!, #N/A)', {}, '<Ranges>(A1)=[[#N/A]]'),
         ('A1', '=IF(TRUE, "1a", "2b")', {}, '<Ranges>(A1)=[[\'1a\']]'),
         ('A1', '=IFS(TRUE, "FIRST")', {}, '<Ranges>(A1)=[[\'FIRST\']]'),
-        ('A1', '=IFS(FALSE, "FIRST", TRUE, "SECOND")', {}, '<Ranges>(A1)=[[\'SECOND\']]'),
-        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND", TRUE, "THIRD")', {}, '<Ranges>(A1)=[[\'THIRD\']]'),
-        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND", TRUE,)', {}, '<Ranges>(A1)=[[0]]'),
-        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND")', {}, '<Ranges>(A1)=[[#N/A]]'),
+        ('A1', '=IFS(FALSE, "FIRST", TRUE, "SECOND")', {},
+         '<Ranges>(A1)=[[\'SECOND\']]'),
+        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND", TRUE, "THIRD")', {},
+         '<Ranges>(A1)=[[\'THIRD\']]'),
+        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND", TRUE,)', {},
+         '<Ranges>(A1)=[[0]]'),
+        ('A1', '=IFS(FALSE, "FIRST", FALSE, "SECOND")', {},
+         '<Ranges>(A1)=[[#N/A]]'),
         ('A1', '=ROW(4:7)', inp_ranges('4:7'), '<Ranges>(A1)=[[4]]'),
         ('A1', '=ROW(B8:D8:F7:H8 D7:E8)',
          inp_ranges('B8:D8', 'F7:H8', 'D7:E8'), '<Ranges>(A1)=[[7]]'),
